@@ -1,9 +1,12 @@
 import sys
 from django.forms import ModelForm
+from django.db.models import Q
 from django_currentuser.middleware import get_current_user
 from crispy_forms.helper import FormHelper
-from crispy_forms.layout import Layout, Fieldset, Submit, Div
+from crispy_forms.layout import Layout, Fieldset, Submit, Div, Hidden, HTML
 from crispy_forms.bootstrap import FormActions, AppendedText, InlineCheckboxes, PrependedText, TabHolder, Tab
+from dal import autocomplete
+from bootstrap_datepicker_plus import DatePickerInput, DateTimePickerInput, YearPickerInput, MonthPickerInput
 
 from schema.models import Accessory, Archive, Battery, BulkFilm, Camera, CameraModel, Developer, Enlarger, FilmStock, Filter
 from schema.models import Flash, FlashProtocol, Format, Lens, LensModel, Manufacturer
@@ -25,6 +28,10 @@ class AccessoryForm(ModelForm):
             'camera_model_compatibility',
             'lens_model_compatibility',
         ]
+        widgets = {
+            'acquired': DatePickerInput(format='%Y-%m-%d'),
+            'lost': DatePickerInput(format='%Y-%m-%d'),
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
             fields.remove('camera_model_compatibility')
@@ -88,6 +95,10 @@ class BulkFilmForm(ModelForm):
             'batch',
             'expiry',
         ]
+        widgets = {
+            'purchase_date': DatePickerInput(format='%Y-%m-%d'),
+            'expiry': MonthPickerInput(format='%Y-%m-01'),
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('filmstock')
             fields.remove('format')
@@ -102,6 +113,10 @@ class CameraForm(ModelForm):
     class Meta:
         model = Camera
         fields = '__all__'
+        widgets = {
+            'acquired': DatePickerInput(format='%Y-%m-%d'),
+            'lost': DatePickerInput(format='%Y-%m-%d'),
+        }
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -127,10 +142,16 @@ class CameraForm(ModelForm):
             )
         )
 
-class CameraModelForm(ModelForm):
+
+class CameraModelForm(autocomplete.FutureModelForm):
     class Meta:
         model = CameraModel
         fields = '__all__'
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete'),
+            'introduced': YearPickerInput(format='%Y'),
+            'discontinued': YearPickerInput(format='%Y'),
+        }
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -255,6 +276,8 @@ class CameraModelForm(ModelForm):
                 'tags',
                 'url',
                 'image',
+                'image_attribution',
+                'image_attribution_url',
             ),
             FormActions(
                 Submit('save', 'Save')
@@ -273,6 +296,9 @@ class DeveloperForm(ModelForm):
             'chemistry',
             'tags',
         ]
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete')
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
 
@@ -298,6 +324,12 @@ class EnlargerForm(ModelForm):
             'cost',
             'lost_price',
         ]
+        widgets = {
+            'acquired': DatePickerInput(format='%Y-%m-%d'),
+            'lost': DatePickerInput(format='%Y-%m-%d'),
+            'introduced': YearPickerInput(format='%Y'),
+            'discontinued': YearPickerInput(format='%Y'),
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
             fields.remove('negative_size')
@@ -320,6 +352,9 @@ class FilmStockForm(ModelForm):
             'process',
             'tags',
         ]
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete')
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
             fields.remove('process')
@@ -370,6 +405,9 @@ class FlashForm(ModelForm):
             'acquired',
             'cost',
         ]
+        widgets = {
+            'acquired': DatePickerInput(format='%Y-%m-%d'),
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
             fields.remove('flash_protocol')
@@ -431,6 +469,10 @@ class LensForm(ModelForm):
             'condition',
             'condition_notes',
         ]
+        widgets = {
+            'acquired': DatePickerInput(format='%Y-%m-%d'),
+            'lost': DatePickerInput(format='%Y-%m-%d'),
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('lensmodel')
 
@@ -444,6 +486,11 @@ class LensModelForm(ModelForm):
     class Meta:
         model = LensModel
         fields = '__all__'
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete'),
+            'introduced': YearPickerInput(format='%Y'),
+            'discontinued': YearPickerInput(format='%Y'),
+        }
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -494,7 +541,11 @@ class LensModelForm(ModelForm):
                 'tags',
                 'url',
                 'image',
+                'image_attribution',
+                'image_attribution_url',
                 'diagram',
+                'diagram_attribution',
+                'diagram_attribution_url',
             ),
             FormActions(
                 Submit('save', 'Save')
@@ -514,6 +565,9 @@ class ManufacturerForm(ModelForm):
             'dissolved',
             'tags',
         ]
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete')
+        }
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -533,6 +587,9 @@ class MountForm(ModelForm):
             'manufacturer',
             'tags',
         ]
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete')
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
 
@@ -614,6 +671,9 @@ class PaperStockForm(ModelForm):
             'finish',
             'tags',
         ]
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete')
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
 
@@ -644,7 +704,7 @@ class PrintForm(ModelForm):
         self.fields['lens'].queryset = Lens.objects.filter(
             owner=get_current_user())
         self.fields['archive'].queryset = Archive.objects.filter(
-            owner=get_current_user())
+            owner=get_current_user(), type='Print', sealed=False)
         self.fields['printer'].queryset = Person.objects.filter(
             owner=get_current_user())
         self.helper = FormHelper(self)
@@ -674,6 +734,9 @@ class PrintForm(ModelForm):
             'archive',
             'printer',
         ]
+        widgets = {
+            'date': DatePickerInput(format='%Y-%m-%d'),
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('paper_stock')
             fields.remove('developer')
@@ -780,9 +843,22 @@ class NegativeForm(ModelForm):
             'photographer',
             'copy_of',
         ]
+        widgets = {
+            'date': DateTimePickerInput(format='%Y-%m-%d %H:%M'),
+        }
 
 
 class FilmForm(ModelForm):
+    class Meta:
+        model = Film
+        fields = '__all__'
+        widgets = {
+            'date_loaded': DatePickerInput(format='%Y-%m-%d'),
+            'date_processed': DatePickerInput(format='%Y-%m-%d'),
+            'purchase_date': DatePickerInput(format='%Y-%m-%d'),
+            'expiry_date': MonthPickerInput(format='%Y-%m-01'),
+        }
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.fields['camera'].queryset = Camera.objects.filter(
@@ -794,14 +870,11 @@ class FilmForm(ModelForm):
         self.fields['archive'].queryset = Archive.objects.filter(
             owner=get_current_user())
         self.helper = FormHelper(self)
-        self.helper.layout.append(Submit('Save', 'Save'))
-
-    class Meta:
-        model = Film
-        fields = [
+        self.helper.layout = Layout(
             'filmstock',
             'exposed_at',
             'format',
+            'status',
             'date_loaded',
             'date_processed',
             'camera',
@@ -822,11 +895,144 @@ class FilmForm(ModelForm):
             'price',
             'processed_by',
             'archive',
-        ]
-        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
-            fields.remove('filmstock')
-            fields.remove('developer')
-            fields.remove('format')
+            FormActions(
+                Submit('save', 'Save'),
+            ),
+        )
+
+
+class FilmAddForm(ModelForm):
+    class Meta:
+        model = Film
+        fields = ['filmstock', 'format', 'frames', 'film_batch', 'expiry_date',
+                  'purchase_date', 'price', 'bulk_film', 'bulk_film_loaded', 'status']
+        widgets = {
+            'purchase_date': DatePickerInput(format='%Y-%m-%d'),
+            'expiry_date': MonthPickerInput(format='%Y-%m-01'),
+        }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.fields['bulk_film'].queryset = BulkFilm.objects.filter(
+            owner=get_current_user())
+        self.helper = FormHelper(self)
+        self.helper.layout = Layout(
+            Fieldset(
+                'Add a new film to your collection',
+                'filmstock',
+                'format',
+                'frames',
+                Div(
+                    TabHolder(
+                        Tab('Single film',
+                            HTML(
+                                "<p>Choose Single Film for regular roll or sheet films</p>"),
+                            'film_batch',
+                            'expiry_date',
+                            'purchase_date',
+                            'price',
+                            ),
+                        Tab('Bulk film',
+                            HTML(
+                                "<p>Choose Bulk Film for film that has been cut from a bulk roll</p>"),
+                            'bulk_film',
+                            'bulk_film_loaded',
+                            ),
+                    ),
+                    css_class="border",
+                ),
+            ),
+            Hidden('status', 'Available'),
+            FormActions(
+                Submit('save', 'Save'),
+            ),
+        )
+
+
+class FilmLoadForm(ModelForm):
+    class Meta:
+        model = Film
+        fields = ['camera', 'title', 'exposed_at',
+                  'date_loaded', 'frames', 'status']
+        widgets = {
+            'date_loaded': DatePickerInput(format='%Y-%m-%d'),
+        }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.fields['camera'].queryset = Camera.objects.filter(
+            owner=get_current_user())
+        self.helper = FormHelper(self)
+        self.helper.layout = Layout(
+            Fieldset(
+                'Load this film into a camera',
+                'camera',
+                'title',
+                'exposed_at',
+                'date_loaded',
+                'frames',
+            ),
+            Hidden('status', 'Loaded'),
+            FormActions(
+                Submit('save', 'Save'),
+            ),
+        )
+
+
+class FilmDevelopForm(ModelForm):
+    class Meta:
+        model = Film
+        fields = ['date_processed', 'developer', 'directory', 'dev_uses', 'dev_time',
+                  'dev_temp', 'dev_n', 'development_notes', 'processed_by', 'status']
+        widgets = {
+            'date_processed': DatePickerInput(format='%Y-%m-%d'),
+        }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.fields['processed_by'].queryset = Person.objects.filter(
+            owner=get_current_user())
+        self.helper = FormHelper(self)
+        self.helper.layout = Layout(
+            Fieldset(
+                'Develop this film',
+                'date_processed',
+                'developer',
+                'directory',
+                'dev_uses',
+                'dev_time',
+                AppendedText('dev_temp', '&deg;C'),
+                'dev_n',
+                'development_notes',
+                'processed_by',
+            ),
+            Hidden('status', 'Developed'),
+            FormActions(
+                Submit('save', 'Save'),
+            ),
+        )
+
+
+class FilmArchiveForm(ModelForm):
+    class Meta:
+        model = Film
+        fields = ['archive', 'status']
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.fields['archive'].queryset = Archive.objects.filter(
+            Q(type='Negative') | Q(type='Slide'), owner=get_current_user(), sealed=False)
+        self.helper = FormHelper(self)
+        self.helper.layout = Layout(
+            Fieldset(
+                'Archive this film',
+                'archive',
+            ),
+            Hidden('status', 'Archived'),
+            FormActions(
+                Submit('save', 'Save'),
+            ),
+        )
 
 
 class TeleconverterForm(ModelForm):
@@ -864,6 +1070,9 @@ class TonerForm(ModelForm):
             'stock_dilution',
             'tags',
         ]
+        widgets = {
+            'tags': autocomplete.TaggitSelect2('tag-autocomplete')
+        }
         if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
             fields.remove('manufacturer')
 
