@@ -15,6 +15,7 @@ urlpatterns = [
 
     path('tag/', views.TagList.as_view(), name='tag-list'),
     path('tag/<slug:slug>', views.TagDetail.as_view(), name='tag-detail'),
+    path('tag-autocomplete/', views.TagAutocomplete.as_view(), name='tag-autocomplete'),
 
     path('accessory/', views.AccessoryList.as_view(), name='accessory-list'),
     path('accessory/<int:id_owner>', views.AccessoryDetail.as_view(),
@@ -211,6 +212,9 @@ urlpatterns = [
     path('film/create/', views.FilmCreate.as_view(), name='film-create'),
     path('film/<int:id_owner>/update',
          views.FilmUpdate.as_view(), name='film-update'),
+    path('film/<int:id_owner>/load', views.FilmLoad.as_view(), name='film-load'),
+    path('film/<int:id_owner>/develop', views.FilmDevelop.as_view(), name='film-develop'),
+    path('film/<int:id_owner>/archive', views.FilmArchive.as_view(), name='film-archive'),
 
     path('teleconverter/', views.TeleconverterList.as_view(),
          name='teleconverter-list'),
