@@ -13,8 +13,8 @@ from django_tables2 import SingleTableView
 from django_tables2.views import SingleTableMixin
 from django_filters.views import FilterView
 from watson.views import SearchMixin
-
 from taggit.models import Tag
+from dal import autocomplete
 
 from schema.models import Accessory, Archive, Battery, BulkFilm, Camera, CameraModel, Developer, Enlarger, FilmStock, Filter
 from schema.models import Flash, FlashProtocol, Format, Lens, LensModel, Manufacturer
@@ -29,7 +29,7 @@ from schema.tables import ProcessTable, RepairTable, ScanTable, NegativeTable, F
 from schema.forms import AccessoryForm, ArchiveForm, BatteryForm, BulkFilmForm, CameraForm, CameraModelForm, DeveloperForm, EnlargerForm, FilmStockForm, FilterForm
 from schema.forms import FlashForm, FlashProtocolForm, FormatForm, LensForm, LensModelForm, ManufacturerForm
 from schema.forms import MountForm, MountAdapterForm, NegativeSizeForm, OrderForm, PaperStockForm, PersonForm, PrintForm
-from schema.forms import ProcessForm, RepairForm, ScanForm, NegativeForm, FilmForm, TeleconverterForm, TonerForm
+from schema.forms import ProcessForm, RepairForm, ScanForm, NegativeForm, FilmForm, FilmAddForm, FilmLoadForm, FilmDevelopForm, FilmArchiveForm, TeleconverterForm, TonerForm
 
 from schema.filters import AccessoryFilter, BatteryFilter, BulkFilmFilter, CameraFilter, CameraModelFilter, DeveloperFilter
 from schema.filters import EnlargerFilter, FilmFilter, FilmStockFilter, FlashFilter, LensFilter, LensModelFilter
@@ -1000,7 +1000,7 @@ class FilmDetail(LoginRequiredMixin, generic.DetailView):
 
 class FilmCreate(LoginRequiredMixin, CreateView):
     model = Film
-    form_class = FilmForm
+    form_class = FilmAddForm
     template_name = 'create.html'
 
 
@@ -1014,6 +1014,36 @@ class FilmUpdate(LoginRequiredMixin, UpdateView):
         return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])
 
 
+class FilmLoad(LoginRequiredMixin, UpdateView):
+    model = Film
+    form_class = FilmLoadForm
+    template_name = 'update.html'
+
+    # Restrict to objects we own
+    def get_object(self):
+        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])
+
+
+class FilmDevelop(LoginRequiredMixin, UpdateView):
+    model = Film
+    form_class = FilmDevelopForm
+    template_name = 'update.html'
+
+    # Restrict to objects we own
+    def get_object(self):
+        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])
+
+
+class FilmArchive(LoginRequiredMixin, UpdateView):
+    model = Film
+    form_class = FilmArchiveForm
+    template_name = 'update.html'
+
+    # Restrict to objects we own
+    def get_object(self):
+        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])
+
+
 class TeleconverterList(LoginRequiredMixin, PagedFilteredTableView):
     model = Teleconverter
     table_class = TeleconverterTable
@@ -1280,3 +1310,17 @@ class TagDetail(generic.DetailView):
 
         context['taggeditems'] = items
         return context
+
+
+class TagAutocomplete(autocomplete.Select2QuerySetView):
+    def get_queryset(self):
+        # Don't forget to filter out results depending on the visitor !
+        if not self.request.user.is_authenticated:
+            return Tag.objects.none()
+
+        qs = Tag.objects.all().order_by('name')
+
+        if self.q:
+            qs = qs.filter(name__istartswith=self.q)
+
+        return qs
